#!/bin/bash

DEBUG_PRINT=0

SCRIPT_VERSION="1.50"
ROM_IMAGE_HEADER="BOOT1 BOOT2 TTX_FPGA"

BASE_DIR=/usr/local/teletimex/update
ROM_DIR=$BASE_DIR/rom
FLASH_UTIL_DIR=/usr/local/ttx/bin/
LOG_DIR=/usr/local/ttx/log
TMP_DIR=/tmp/ttx

#----
FLASH_UPDATE=$FLASH_UTIL_DIR/flash_update
FLASH_DUMP=$FLASH_UTIL_DIR/flash_dump
FLASH_DUMP_IMG=$TMP_DIR/flash_dump.bin
ROM_UPDATE_LOG=$LOG_DIR/rom_update.log

MAX_LOG_LINE=300
# ROM string offset
ROM_NO_OFFSET_FROM_TAIL=54

# Flash sector
BOOT1_SECTOR=0
BOOT2_SECTOR=8
TTX_FPGA_SECTOR=18


trap '' INT TERM SIGTSTP

# debug echo
decho()
{
	if [ $DEBUG_PRINT = 1 ]; then
		echo -e $1 | tee -a $ROM_UPDATE_LOG 2>&1
	fi
}


# Get ROM number string from FlashROM image file.
# set IMAGE_ROM_NO
get_image_rom_number()
{
#	image_name=$ROM_DIR/$1
	fullpath_name=$ROM_DIR/$1

	size=`ls -l $fullpath_name | cut -f 5 -d " "`
	rom_offset=`expr $size - $ROM_NO_OFFSET_FROM_TAIL`

	image_rom_no_tmp=`dd if=$fullpath_name skip=$rom_offset bs=1 count=8 2> /dev/null`
	if [ $? -ne 0 ]; then
		echo "[Error] get_image_rom_number" | tee -a $ROM_UPDATE_LOG 2>&1
		return 1
	fi
	IMAGE_ROM_NO=`echo -n  $image_rom_no_tmp | sed -e s/[0-9][0-9][0-9][0-9]//`

	return 0
}

# Get ROM number.
get_flash_rom_number()
{
	# Get image sector & ROM number.
	basename=`echo  $1 | LANG=C perl -pe 's/(.*)_[0-9]*.bin$/$1/'`

	case $basename in
		"BOOT1" ) flash_offset=8138
			;;
		"BOOT2" ) flash_offset=262090
			;;
		"TTX_FPGA" ) flash_offset=982986
			;;
		* )
			echo "[Error] Invalid ROM image file name." | tee -a $ROM_UPDATE_LOG 2>&1
			return 1
			;;
	esac

	flash_rom_no=`dd if=$FLASH_DUMP_IMG skip=$flash_offset bs=1 count=8 2> /dev/null`
	if [ $? -ne 0 ]; then
		echo "[Error] get_flash_rom_number " | tee -a $ROM_UPDATE_LOG 2>&1
		return 1
	fi

	FLASH_ROM_NUMBER=`echo -n  $flash_rom_no | sed -e s/[0-9][0-9][0-9][0-9]//`

	return 0
}

# FlsahROM image is compared with ROM dump image.
compare_version()
{

	decho "IMAGE_VERSION" $IMAGE_VERSION
	decho "FLASH_VERSION" $FLASH_VERSION

	if [ `expr $IMAGE_VERSION` -gt `expr $FLASH_VERSION` ]; then
		# ROM image > FlashROM
		# update YES
		return 0
	else
		# ROM image == FlashROM
		# update NO
		return 1
	fi
}

# Create real FlashROM image file by flash_dump(elf binary).
create_dump_image()
{
	ls $FLASH_DUMP > /dev/null 2>&1
	if [ $? -ne 0 ]; then
		echo "[Error] flash_dump is not faund. " | tee -a $ROM_UPDATE_LOG 2>&1
		return 1
	fi
	$FLASH_DUMP $FLASH_DUMP_IMG> /dev/null 2>&1
	decho "Created flash_dump.bin"

	return 0
}

# Print FlashROM set version.
get_flash_version()
{
	create_dump_image
	if [ $? -ne 0 ]; then
		echo "[Error] create_dump_image"
		return 1
	fi

#	val=`ls -1 $ROM_DIR | grep BOOT1`
#	get_flash_rom_number $val
	get_flash_rom_number "BOOT1"
	if [ $? -ne 0 ]; then
		echo "[Error] get BOOT version"
		return 1
	fi
	boot1_number=$FLASH_ROM_NUMBER

#	val=`ls -1 $ROM_DIR | grep BOOT2`
#	get_flash_rom_number $val
	get_flash_rom_number "BOOT2"
	if [ $? -ne 0 ]; then
		echo "[Error] get BOOT version"
		return 1
	fi
	boot2_number=$FLASH_ROM_NUMBER

#	val=`ls -1 $ROM_DIR | grep TTX_FPGA`
#	get_flash_rom_number $val
	get_flash_rom_number TTX_FPGA
	if [ $? -ne 0 ]; then
		echo "[Error] get TTX_FPGA version"
		return 1
	fi
	ttx_fpga_number=$FLASH_ROM_NUMBER

	boot1_tmp=`expr $boot1_number \* 10000`
	boot2_tmp=`expr $boot2_number \* 100`
	ttx_fpga_tmp=`expr $ttx_fpga_number`
	FLASH_VERSION=`expr $boot1_tmp + $boot2_tmp + $ttx_fpga_tmp`
	decho "flash version:"$FLASH_VERSION

	PRINT_FLASH_VERSION=`echo "$boot1_number.$boot2_number.0$ttx_fpga_number"`

	if [ `basename $0` == "get_flash_version.sh" ]; then
		date	>> $ROM_UPDATE_LOG 2>&1
		echo $0	>> $ROM_UPDATE_LOG 2>&1
		echo "==>" $PRINT_FLASH_VERSION	>> $ROM_UPDATE_LOG 2>&1
		echo "------------------------------------------" >> $ROM_UPDATE_LOG 2>&1
	fi

	rm -rf $FLASH_DUMP_IMG

	return 0
}



# Judgement FlashROM update.
check_flash_update()
{
	# Get Flash ROM version.
	get_flash_version
	if [ $? -ne 0 ]; then
		echo "[Error] get_flash_version()"
		return 1
	fi

	# Get write ROM image Version.
	image=`ls -1 $ROM_DIR | grep BOOT1`
	get_image_rom_number $image
	if [ $? -ne 0 ]; then
		return 1
	fi
	boot1_number=$IMAGE_ROM_NO

	image=`ls -1 $ROM_DIR | grep BOOT2`
	get_image_rom_number $image
	if [ $? -ne 0 ]; then
		return 1
	fi
	boot2_number=$IMAGE_ROM_NO

	image=`ls -1 $ROM_DIR | grep TTX_FPGA`
	get_image_rom_number $image
	if [ $? -ne 0 ]; then
		return 1
	fi
	ttx_fpga_number=$IMAGE_ROM_NO

	boot1_tmp=`expr $boot1_number \* 10000`
	boot2_tmp=`expr $boot2_number \* 100`
	ttx_fpga_tmp=`expr $ttx_fpga_number`
	IMAGE_VERSION=`expr $boot1_tmp + $boot2_tmp + $ttx_fpga_tmp`

	decho "image version: Ver $boot1_number.$boot2_number.0$ttx_fpga_number"
	decho "image version:"$IMAGE_VERSION


	date	>> $ROM_UPDATE_LOG 2>&1
	echo $0	>> $ROM_UPDATE_LOG 2>&1
	echo -n "==> ROM image: " $IMAGE_VERSION ", FlashROM: " $FLASH_VERSION ", " >> $ROM_UPDATE_LOG 2>&1

	# Compare ROM number.
	compare_version
	if [ $? -ne 0 ]; then
		echo " update=NO" >> $ROM_UPDATE_LOG 2>&1
		echo "------------------------------------------" >> $ROM_UPDATE_LOG 2>&1
		return 1
	else
		echo " update=YES" >> $ROM_UPDATE_LOG 2>&1
		echo "------------------------------------------" >> $ROM_UPDATE_LOG 2>&1
		return 0
	fi
}

# FlashROM image set update.
do_flash_update()
{
	result=0

	date	>> $ROM_UPDATE_LOG 2>&1
	echo $0	>> $ROM_UPDATE_LOG 2>&1

	image=`ls -1 $ROM_DIR | grep BOOT1`
	$FLASH_UPDATE $BOOT1_SECTOR $ROM_DIR/$image | tee -a $ROM_UPDATE_LOG 2>&1
	if [ $? -ne 0 ]; then
		echo "[Error] $image write failed !!!"  | tee -a $ROM_UPDATE_LOG 2>&1
		echo "==> Failed" | tee -a $ROM_UPDATE_LOG 2>&1
		result=1
	else
		echo "==> Success" | tee -a $ROM_UPDATE_LOG 2>&1
	fi
	image=`ls -1 $ROM_DIR | grep BOOT2`
	$FLASH_UPDATE $BOOT2_SECTOR $ROM_DIR/$image | tee -a $ROM_UPDATE_LOG 2>&1
	if [ $? -ne 0 ]; then
		echo "[Error] $image write failed !!!"  | tee -a $ROM_UPDATE_LOG 2>&1
		echo "==> Failed" | tee -a $ROM_UPDATE_LOG 2>&1
		result=1
	else
		echo "==> Success" | tee -a $ROM_UPDATE_LOG 2>&1
	fi
	image=`ls -1 $ROM_DIR | grep TTX_FPGA`
	$FLASH_UPDATE $TTX_FPGA_SECTOR $ROM_DIR/$image | tee -a $ROM_UPDATE_LOG 2>&1
	if [ $? -ne 0 ]; then
		echo "[Error] $image write failed !!!"  | tee -a $ROM_UPDATE_LOG 2>&1
		echo "==> Failed" | tee -a $ROM_UPDATE_LOG 2>&1
		result=1
	else
		echo "==> Success" | tee -a $ROM_UPDATE_LOG 2>&1
	fi
	echo "------------------------------------------" >> $ROM_UPDATE_LOG 2>&1

	return $result
}

# switch script
script_router()
{
	if [ \( `basename $0` == "get_flash_version_updt_app.sh" \) -o \
		 \( `basename $0` == "get_flash_version_updt_rom.sh" \) ]; then
		# ROM バージョンを取得し stdout に出力する。
		get_flash_version
		if [ $? -ne 0 ]; then
			echo "[Error] get_flash_version"
			return 1
		else
			echo $PRINT_FLASH_VERSION
			return 0
		fi
	elif [ `basename $0` == "check_flash_update.sh" ]; then
		# 更新の必要性をチェック。0:更新する, 0以外:更新しない。
		decho "check_flash_update"
		check_flash_update
		if [ $? -ne 0 ]; then
			decho "[Success] unwanted update"
			return 1
		else
			decho "[Success] need update !!"
			return 0
		fi
	elif [ `basename $0` == "do_flash_update.sh" ]; then

		ls $BASE_DIR/DUMMY_WRITE > /dev/null 2>&1
		if [ $? -ne 0 ]; then
			# 更新を実行する。0:完了, 0以外:失敗。
			do_flash_update
		else
			echo dry run FlashROM write.
		fi

		if [ $? -ne 0 ]; then
			echo "[Error] do_flash_update"
			return 1
		else
			decho "[Success] do_flash_update"
			return 0
		fi
	else
		echo "[Error] invalid script name"
		return 1
	fi
}


## entry point

# module auto load
lsmod | grep ttx_flash > /dev/null 2>&1
if [ $? -ne 0 ]; then
	sudo modprobe ttx-flash
fi

# print version
if [ $# -eq 1 ]; then
	if [ $1 == "--version" ]; then
		echo $SCRIPT_VERSION
		exit 0
	else
		echo "[Error] Argument val"
		exit 1
	fi
fi

script_router
if [ $? -ne 0 ]; then
	exit 1
else
	exit 0
fi
